@@ -18,8 +18,7 @@ from pymich.middle_end.ir.instr_types import Mutez, base_types
 from pymich.backend.micheline_emitter import CompilerBackend
 from pymich.frontend.compiler import python_to_ir
 from pymich.utils.helpers import ast_to_tree
-from pymich.middle_end.ir.vm_types import (Contract, Entrypoint, FunctionPrototype, Instr,
-                             MultiArgFunctionPrototype)
+from pymich.middle_end.ir.vm_types import (Contract, Entrypoint, Instr)
 import pymich.utils.exceptions as E
 
 from pymich.utils.compiler_stdlib import *
@@ -542,21 +541,13 @@ class Compiler:
             return []
         value_var_name = attribute.value.id
         value_type = e.types[value_var_name]
-        attribute_name = attribute.attr
         if type(value_type) in self.std_lib_types:
-            value_methods = self.std_lib_types[type(value_type)].methods
-            if attribute_name in value_methods:
-                method_prototype = value_methods[attribute_name].get_prototype(value_type)
-                compile_callback = value_methods[attribute_name].compile_callback
-                method_instance = t.StdlibMethodInstance(
-                    assign_var_name,
-                    method_prototype,
-                    compile_callback,
-                )
-                instr = self._fetch_variable(value_var_name, e)
-                e.types[assign_var_name] = method_instance
-                e.vars[assign_var_name] = e.sp
-                return instr
+            instr = self._fetch_variable(value_var_name, e)
+            method_type = self.type_checker.get_expression_type(attribute, e)
+            method_type.var_name = assign_var_name
+            e.types[assign_var_name] = method_type
+            e.vars[assign_var_name] = e.sp
+            return instr
         else:
             return []
 
@@ -846,8 +837,8 @@ class Compiler:
             e,
         )
 
-    def _get_function_prototype(self, f: ast.FunctionDef, e: Env) -> FunctionPrototype:
-        return FunctionPrototype(
+    def _get_function_prototype(self, f: ast.FunctionDef, e: Env) -> t.FunctionPrototype:
+        return t.FunctionPrototype(
             self.type_parser.parse(f.args.args[0].annotation, e, lineno=f.args.args[0].lineno),
             self.type_parser.parse(f.returns, e, lineno=f.returns.lineno if f.returns else 0),
         )
@@ -988,18 +979,10 @@ class Compiler:
             actual_type = self.type_checker.get_expression_type(arg, e)
             expected_type = e.types[c.func.id].attribute_types[i]
 
-            # TODO solve callable vs fnuctionprototype casting
-            if type(expected_type) == t.Callable:
-                expected_type = FunctionPrototype(expected_type.param_type, expected_type.return_type)
-
-            # TODO solve callable vs fnuctionprototype casting
-            if type(actual_type) == t.Callable:
-                actual_type = FunctionPrototype(actual_type.param_type, actual_type.return_type)
-
             if type(actual_type) == type(expected_type) == t.Dict and actual_type.key_type == t.Unknown():
                 pass
             elif actual_type != expected_type:
-                raise E.FunctionParameterTypeException(c.func.id, expected_type, actual_type, c.args[0].lineno)
+                raise E.FunctionParameterTypeException(expected_type, actual_type, c.args[0].lineno)
 
         instructions = RecordUtils.compile_record(e.types[c.func.id], c.args, self._compile, e)
         return instructions
@@ -1232,8 +1215,7 @@ class Compiler:
             instr =  self.compile_std_lib_method_instance_call(f, e)
             # edge case since Bytes.unpack takes the return type as argument
             if (
-                not isinstance(e.types[f.func.id].signature, MultiArgFunctionPrototype)
-                and isinstance(e.types[f.func.id].signature.arg_type, t.Bytes)
+                isinstance(e.types[f.func.id].signature.arg_types[0], t.Bytes)
             ):
                 cast_type = self.type_parser.parse(f.args[0], e)
                 e.types[f.func.id].signature.return_type = cast_type
@@ -1246,12 +1228,12 @@ class Compiler:
         # typecheck argtype
         actual_type = self.type_checker.get_expression_type(f.args[0], e)
 
-        if type(prototype) == FunctionPrototype:
+        if type(prototype) == t.FunctionPrototype:
             expected_type = prototype.arg_type
         else:
             expected_type = prototype.param_type
         if actual_type != expected_type:
-            raise E.FunctionParameterTypeException(f.func.id, expected_type, actual_type, f.args[0].lineno)
+            raise E.FunctionParameterTypeException(expected_type, actual_type, f.args[0].lineno)
 
         func_addr = e.vars[function_name]
         jump_length = e.sp - func_addr
