@@ -12,8 +12,6 @@ import pymich.utils.exceptions as E
 
 
 def simplify_type(el_type):
-    if isinstance(el_type, (t.FunctionPrototype, vm_types.FunctionPrototype)):
-        return t.Callable(simplify_type(el_type.arg_type), simplify_type(el_type.return_type))
     if isinstance(el_type, t.Record):
         return simplify_type(el_type.get_type())
     if isinstance(el_type, t.Pair):
@@ -1343,7 +1341,7 @@ class StackManipulationTypeChecker(TypeChecker):
         function_stack.assert_length(1, "The body of a LAMBDA must return a stack of 1 element")
         function_stack.assert_top_type([return_type], "LAMBDA body does not return type matching its signature.")
 
-        self.stack.push_type(t.Callable(arg_type, return_type, function_body))
+        self.stack.push_type(t.FunctionPrototype(arg_type, return_type, function_body))
 
     def typecheck_dig(self, instruction: Instr):
         if not len(instruction.args):
@@ -1457,9 +1455,8 @@ class ControlStructuresTypeChecker(TypeChecker):
         self.stack.assert_min_length(2, "LAMBDA requires a stack of minimum length 2.")
         arg_type = simplify_type(self.stack.pop())
         lambda_type = simplify_type(self.stack.pop())
-        param_type = simplify_type(simplify_type(lambda_type).param_type)
-        if arg_type != param_type:
-            raise E.LambdaArgumentTypeException(arg_type, param_type)
+        if arg_type != lambda_type.arg_type:
+            raise E.LambdaArgumentTypeException(arg_type, arg_type)
         self.stack.push_type(lambda_type.return_type)
 
     def typecheck_apply(self):
@@ -1471,19 +1468,19 @@ class ControlStructuresTypeChecker(TypeChecker):
         stack_exception = E.StackTopTypeException(
             "APPLY",
             [t.Contract(t.TypeVar()), t.Mutez(), t.TypeVar()],
-            [t.TypeVar("T"), t.Callable(t.Pair(t.TypeVar("T"), t.TypeVar("U")), t.TypeVar("V"))],
+            [t.TypeVar("T"), t.FunctionPrototype(t.Pair(t.TypeVar("T"), t.TypeVar("U")), t.TypeVar("V"))],
             [partial_arg_type, lambda_type],
         )
-        if not isinstance(lambda_type, t.Callable) or not isinstance(lambda_type.param_type, t.Record):
+        if not isinstance(lambda_type, t.FunctionPrototype) or not isinstance(lambda_type.arg_type, t.Record):
             raise stack_exception
 
-        arg_type = lambda_type.param_type.get_type()
+        arg_type = lambda_type.arg_type.get_type()
         appliable_arg_type = simplify_type(arg_type.car)
 
         if appliable_arg_type != partial_arg_type:
             raise stack_exception
         else:
-            lambda_type.param_type = arg_type.cdr
+            lambda_type.arg_type = arg_type.cdr
             lambda_type.applied_arg_types.append(partial_arg_type)
             self.stack.push_type(lambda_type)
 
