@@ -1,23 +1,27 @@
 import ast
-from dataclasses import dataclass
+from dataclasses import dataclass, field
 from copy import deepcopy
-from typing import Optional, List
+from typing import Optional, Sequence
 import pymich.utils.exceptions as E
 
 
 class Type:
     def __init__(self, annotation: Optional[str] = None):
+        self.__class__.__hash__ = Type.__hash__
         self.annotation = annotation
-        pass
 
     def __eq__(self, o):
         return type(self) == type(o)
 
     def python_name(self):
-        return self.__str__()
+        return self.__repr__()
+
+    def __hash__(self):
+        return hash(self.__repr__())
+
 
 class Unknown(Type):
-    def __str__(self):
+    def __repr__(self):
         return "unknown"
 
 class TypeVar(Type):
@@ -25,7 +29,7 @@ class TypeVar(Type):
         super().__init__(annotation)
         self.name = name
 
-    def __str__(self):
+    def __repr__(self):
         return self.name
 
 class Option(Type):
@@ -33,7 +37,7 @@ class Option(Type):
         super().__init__(annotation)
         self.option_type = option_type
 
-    def __str__(self):
+    def __repr__(self):
         return f"{self.option_type} option"
 
     def __eq__(self, o):
@@ -46,7 +50,7 @@ class Or(Type):
         self.left_type = left_type
         self.right_type = right_type
 
-    def __str__(self):
+    def __repr__(self):
         return f"or {self.left_type} {self.right_type}"
 
 
@@ -56,10 +60,10 @@ class Union:
     Indeed, in Michelson, `Or` is a type itself and thus `Int` or `Nat` will not be
     a acceptable types if is the expected type `Or(Int, Nat)`.
     """
-    def __init__(self, allowed_types: List[Type]):
+    def __init__(self, allowed_types: Sequence[Type]):
         self.allowed_types = allowed_types
 
-    def __str__(self):
+    def __repr__(self):
         if not len(self.allowed_types):
             return "()"
 
@@ -70,55 +74,57 @@ class Union:
 
 
 class Unit(Type):
-    def __str__(self):
+    def __repr__(self):
         return "unit"
 
 
 class Bytes(Type):
-    def __str__(self):
+    def __repr__(self):
         return "bytes"
 
 class PythonInt(Type):
-    def __str__(self):
+    def __repr__(self):
         return "python_int"
 
 class Int(Type):
-    def __str__(self):
+    def __repr__(self):
         return "int"
 
     def __eq__(self, o):
         return type(self) == type(o)
 
 class Datetime(Type):
-    def __str__(self):
+    def __repr__(self):
         return "timestamp"
 
 
 class Nat(Type):
-    def __str__(self):
+    def __repr__(self):
         return "nat"
 
 
 class Mutez(Type):
-    def __str__(self):
+    def __repr__(self):
         return "mutez"
 
+
 class PythonString(Type):
-    def __str__(self):
+    def __repr__(self):
         return "python_string"
 
+
 class String(Type):
-    def __str__(self):
+    def __repr__(self):
         return "string"
 
 
 class Bool(Type):
-    def __str__(self):
+    def __repr__(self):
         return "bool"
 
 
 class Address(Type):
-    def __str__(self):
+    def __repr__(self):
         return "address"
 
     def python_name(self):
@@ -126,7 +132,7 @@ class Address(Type):
 
 
 class Universal(Type):
-    def __str__(self):
+    def __repr__(self):
         return "universal"
 
     def __eq__(self, o: Type):
@@ -134,7 +140,7 @@ class Universal(Type):
 
 
 class Operation(Type):
-    def __str__(self):
+    def __repr__(self):
         return "operation"
 
 
@@ -153,7 +159,7 @@ class Pair(Type):
             return self == o.get_type()
         return type(self) == type(o) and self.car == o.car and self.cdr == o.cdr
 
-    def __str__(self):
+    def __repr__(self):
         if isinstance(self.car, Pair) and isinstance(self.cdr, Pair):
             return f"pair({self.car.car}, {self.car.cdr}, {self.cdr.car}, {self.cdr.cdr})"
         if isinstance(self.car, Pair):
@@ -164,7 +170,7 @@ class Pair(Type):
             return f"pair({self.car}, {self.cdr})"
 
     @staticmethod
-    def pair_n(args: List[Type]) -> "Pair":
+    def pair_n(args: Sequence[Type]) -> "Pair":
         args.reverse()
         right_comb = Pair(args[1], args[0])
 
@@ -231,7 +237,7 @@ class Pair(Type):
 
 
 class Record(Type):
-    def __init__(self, name: str, attribute_names: List[str], attribute_types: List[Type], annotation: Optional[str] = None):
+    def __init__(self, name: str, attribute_names: Sequence[str], attribute_types: Sequence[Type], annotation: Optional[str] = None):
         super().__init__(annotation)
         self.name = name
         self.attribute_names = attribute_names
@@ -242,7 +248,7 @@ class Record(Type):
             return self.get_type() == o
         return type(self) == type(o) and self.attribute_names == o.attribute_names and self.attribute_types == o.attribute_types
 
-    def __str__(self):
+    def __repr__(self):
         return str(self.get_type())
 
     def make_node(self, left, right):
@@ -277,7 +283,7 @@ class List(Type):
             return self.element_type == o.element_type
         return type(self) == type(o) and self.element_type == o.element_type
 
-    def __str__(self):
+    def __repr__(self):
         return f"{self.element_type} list"
 
 class Operations(Type):
@@ -285,7 +291,7 @@ class Operations(Type):
         self.element_type = Operation()
         super().__init__(annotation)
 
-    def __str__(self):
+    def __repr__(self):
         return f"operation list"
 
     def __eq__(self, o):
@@ -301,7 +307,7 @@ class Set(Type):
     def __eq__(self, o):
         return type(self) == type(o) and self.element_type == o.element_type
 
-    def __str__(self):
+    def __repr__(self):
         return f"{self.element_type} set"
 
 class Spread(Type):
@@ -322,7 +328,7 @@ class Dict(Type):
     def __eq__(self, o):
         return type(self) == type(o) and self.key_type == o.key_type and self.value_type == o.value_type
 
-    def __str__(self):
+    def __repr__(self):
         return f"({self.key_type}, {self.value_type}) map"
 
 
@@ -335,34 +341,38 @@ class BigMap(Type):
     def __eq__(self, o):
         return type(self) == type(o) and self.key_type == o.key_type and self.value_type == o.value_type
 
-    def __str__(self):
+    def __repr__(self):
         return f"({self.key_type}, {self.value_type}) big_map"
 
 
-class Callable(Type):
-    def __init__(self, param_type: Type, return_type: Type, annotation: Optional[str] = None):
-        super().__init__(annotation)
-        self.param_type = param_type
-        self.return_type = return_type
-        self.applied_arg_types = []
-
-    def __eq__(self, o):
-        return type(self) == type(o) and self.param_type == o.param_type and self.return_type == o.return_type
-
-
-#from pymich.compiler_stdlib import StdlibFunctionSpec
-
 class StdlibVariable:
     def __init__(self, type_: Type, compile_callback):
         self.type = type_
         self.compile_callback = compile_callback
 
 
+class FunctionPrototype(Type):
+    """User defined functions"""
+    def __init__(self, arg_type: Type, return_type: Type, annotation: Optional[str] = None):
+        super().__init__(annotation)
+        self.arg_type = arg_type
+        self.return_type = return_type
+        self.applied_arg_types = []
+
+    def __eq__(self, o):
+        return type(self) == type(o) and self.arg_type == o.arg_type and self.return_type == o.return_type
+
+    def __repr__(self):
+        return f"({self.arg_type}) -> {self.return_type}"
+
+
 @dataclass
-class FunctionPrototype:
-    arg_type: Type
+class MultiArgFunctionPrototype(Type):
+    """Standard library functions"""
+    arg_types: Sequence[Type]
     return_type: Type
 
+
 class StdlibFunctionSpec:
     def __init__(self, compile_callback, get_prototype: FunctionPrototype):
         self.compile_callback = compile_callback
@@ -399,7 +409,7 @@ class StdlibMethodInstance(Type):
         self,
         var_name: str,
         signature: StdlibFunctionSpec,
-        compile_callback: Callable,
+        compile_callback: FunctionPrototype,
         annotation: Optional[str] = None,
     ):
         super().__init__(annotation)
@@ -418,7 +428,7 @@ class StdlibStaticMethodInstance(Type):
     def __init__(
         self,
         signature: StdlibFunctionSpec,
-        compile_callback: Callable,
+        compile_callback: FunctionPrototype,
         annotation: Optional[str] = None,
     ):
         super().__init__(annotation)
@@ -438,7 +448,7 @@ class Contract(Type):
     def __eq__(self, o):
         return type(self) == type(o) and self.param_type == o.param_type
 
-    def __str__(self):
+    def __repr__(self):
         return f"{self.param_type} contract"
 
 
@@ -514,7 +524,7 @@ class TypeParser:
     def parse_callable(self, dictionary: ast.Subscript, e, annotation: Optional[str] = None):
         key_type = self.parse(dictionary.slice.elts[0].elts[0], e)
         value_type = self.parse(dictionary.slice.elts[1], e)
-        return Callable(key_type, value_type, annotation)
+        return FunctionPrototype(key_type, value_type, annotation)
 
     def parse_contract(self, contract: ast.Subscript, e, annotation):
         param_type = self.parse(contract.slice, e)
@@ -550,6 +560,13 @@ class TypeParser:
     def parse(self, type_ast, e, annotation: Optional[str] = None, lineno = 0) -> Type:
         if type(type_ast) == ast.Name:
             return self.parse_name(type_ast, e, annotation, lineno)
+        if type(type_ast) == ast.Constant:
+            if isinstance(type_ast.value, str):
+                return PythonString()
+            elif isinstance(type_ast.value, int):
+                return PythonInt()
+            else:
+                raise E.CompilerException(f"Unsupported type: {type_ast}", lineno)
         if type(type_ast) == ast.Subscript:
            return self.parse_subscript(type_ast, e, annotation, lineno)
         if type(type_ast) == ast.Tuple:
