@@ -1,7 +1,6 @@
 import ast
 
 from pymich.middle_end.ir.instr_types import StdlibFunctionSpec, StdLibClassSpec
-from pymich.middle_end.ir.vm_types import FunctionPrototype
 from pymich.utils.compiler_stdlib import StdLib
 import pymich.middle_end.ir.instr_types as t
 from pymich.utils.environment import Env
@@ -80,7 +79,14 @@ class TypeChecker:
         attr_value_type = type(record)
         if attr_value_type in self.std_lib.types_mapping:
             if node.attr in self.std_lib.types_mapping[attr_value_type].methods:
-                return self.std_lib.types_mapping[attr_value_type].methods[node.attr].get_prototype(record).return_type
+                method = self.std_lib.types_mapping[attr_value_type].methods[node.attr]
+                return t.StdlibMethodInstance(
+                    "",
+                    method.get_prototype(record),
+                    method.compile_callback,
+                )
+            else:
+                raise E.CompilerException(f"{node.attr} is not a method of type: {record}", node.lineno)
 
         if node.attr in record.attribute_names:
             return record.get_attribute_type(node.attr)
@@ -96,50 +102,123 @@ class TypeChecker:
             elif isinstance(prototype, (t.StdlibMethodInstance, t.StdlibStaticMethodInstance)):
                 # Stdlib method call
                 return prototype.signature.return_type
-            else:
-                return e.types[node.func.id].return_type
+            elif isinstance(prototype, t.FunctionPrototype):
+                if len(node.args) != 1:
+                    raise E.CompilerException("Only uniary functions are supported at the IR level", node.lineno)
+
+                actual_type = self.get_expression_type(node.args[0], e)
+                expected_type = prototype.arg_type
+                if actual_type != expected_type:
+                    raise E.FunctionParameterTypeException(expected_type, actual_type, node.lineno)
+                return prototype.return_type
         elif type(node.func) == ast.Name and node.func.id in self.std_lib.constants_mapping:
             if type(self.std_lib.constants_mapping[node.func.id]) == StdlibFunctionSpec:
                 return self.std_lib.constants_mapping[node.func.id].get_prototype().return_type
             elif type(self.std_lib.constants_mapping[node.func.id]) == StdLibClassSpec:
                 arg_types = [self.get_expression_type(arg, e) for arg in node.args]
                 prototype = self.std_lib.constants_mapping[node.func.id].constructor.get_prototype(arg_types)
-                if isinstance(prototype, FunctionPrototype):
-                    is_polymorphic = isinstance(prototype.arg_type, t.Unknown)
-                    if arg_types != [prototype.arg_type] and not is_polymorphic:
-                        raise E.TypeException(prototype.arg_type, arg_types, e)
+                if isinstance(prototype, t.MultiArgFunctionPrototype):
+                    is_polymorphic = isinstance(prototype.arg_types[0], t.Unknown)
+                    if arg_types != prototype.arg_types and not is_polymorphic:
+                        raise E.TypeException(prototype.arg_types, arg_types, e)
                 return prototype.return_type
             else:
                 raise E.CompilerException(f"Stdlib expression {node.func.id} cannot be typed", node.lineno)
         elif type(node.func) == ast.Attribute and node.func.attr == "unpack":
+            if len(node.args) != 1:
+                raise E.FunctionParameterTypeException(
+                    "[Type]",
+                    node.args,
+                    node.lineno
+                )
             cast_to_type = self.type_parser.parse(node.args[0], e)
             return self.std_lib.types_mapping[t.Bytes].methods["unpack"].get_prototype(cast_to_type).return_type
         else:
-            return self.get_expression_type(node.func, e)
+            fun_type = self.get_expression_type(node.func, e)
+            if isinstance(fun_type, t.StdlibMethodInstance):
+                is_error = False
+
+                if len(node.args) != len(fun_type.signature.arg_types):
+                    is_error = True
+                else:
+                    for arg, expected_type in zip(node.args, fun_type.signature.arg_types):
+                        actual_type = self.type_parser.parse(arg, e)
+                        if actual_type != expected_type:
+                            is_error = True
+
+                if is_error:
+                    raise E.FunctionParameterTypeException(
+                        fun_type.signature.arg_types,
+                        node.args,
+                        node.lineno
+                    )
+
+                return fun_type.signature.return_type
+            else:
+                return fun_type
 
     def get_compare_type(self, node, e: Env):
         return t.Bool()
 
     def get_binop_type(self, node, e: Env):
+        allowed_operand_types = {
+            ast.Sub: {
+                (t.Nat(), t.Nat()): t.Int(),
+                (t.Int(), t.Int()): t.Int(),
+                (t.Nat(), t.Int()): t.Int(),
+                (t.Int(), t.Nat()): t.Int(),
+                (t.Mutez(), t.Mutez()): t.Option(t.Mutez()),
+                (t.Datetime(), t.Int()): t.Datetime(),
+                (t.Datetime(), t.Datetime()): t.Datetime(),
+            },
+            ast.Add: {
+                (t.Nat(), t.Nat()): t.Nat(),
+                (t.Int(), t.Int()): t.Int(),
+                (t.Nat(), t.Int()): t.Int(),
+                (t.Int(), t.Nat()): t.Int(),
+                (t.Mutez(), t.Mutez()): t.Mutez(),
+                (t.Datetime(), t.Int()): t.Datetime(),
+                (t.Int(), t.Datetime()): t.Datetime(),
+            },
+            ast.Mult: {
+              (t.Nat(), t.Nat()): t.Nat(),
+              (t.Int(), t.Int()): t.Int(),
+              (t.Nat(), t.Int()): t.Int(),
+              (t.Int(), t.Nat()): t.Int(),
+              (t.Mutez(), t.Nat()): t.Mutez(),
+              (t.Nat(), t.Mutez()): t.Mutez(),
+           },
+           ast.FloorDiv: {
+              (t.Nat(), t.Nat()): t.Nat(),
+              (t.Int(), t.Int()): t.Int(),
+              (t.Nat(), t.Int()): t.Int(),
+              (t.Int(), t.Nat()): t.Int(),
+              (t.Mutez(), t.Nat()): t.Mutez(),
+              (t.Mutez(), t.Mutez()): t.Nat(),
+           },
+           ast.Mod: {
+              (t.Nat(), t.Nat()): t.Nat(),
+              (t.Int(), t.Int()): t.Int(),
+              (t.Nat(), t.Int()): t.Int(),
+              (t.Int(), t.Nat()): t.Int(),
+              (t.Mutez(), t.Nat()): t.Mutez(),
+              (t.Mutez(), t.Mutez()): t.Mutez(),
+           },
+        }
+
+        op_type = type(node.op)
+        if op_type not in allowed_operand_types:
+            raise E.CompilerException(f"Unsupported operand ({node.op}, only [+, -, *, %, //] are supported)", node.lineno)
+
         left_type = self.get_expression_type(node.left, e)
         right_type = self.get_expression_type(node.right, e)
-        is_nat_subtraction = (
-            isinstance(left_type, t.Nat)
-            and isinstance(node.op, ast.Sub)
-        )
-        if is_nat_subtraction:
-            return t.Int()
 
-        is_mutez_subtraction = (
-            isinstance(left_type, t.Mutez)
-            and isinstance(node.op, ast.Sub)
-        )
-        if is_mutez_subtraction:
-            return t.Option(t.Mutez())
+        if (left_type, right_type) not in allowed_operand_types[op_type]:
+            raise E.CompilerException(f"Unsupported types for operand {node.op}: [{left_type}, {right_type}]", node.lineno)
 
-        return self.get_expression_type(node.left, e)
+        return allowed_operand_types[op_type][(left_type, right_type)]
 
-    def get_expression_type(self, node, e: Env | None = None):
+    def get_expression_type(self, node, e: Env | None = None) -> t.Type:
         if not e:
             e = Env({}, -1, {}, {})
             self.e = e
